use once_cell::sync::OnceCell;
use serde::{Deserialize, Serialize};
use serde_yaml::{self};


#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct CoreConfig {
    /// Discord bot token.
    pub token: String,
    /// Log Level
    pub log_level: String,
    /// Folder to store program logs.
    pub log_dir: String,
    /// Location for where the sqlite database should be stored
    pub sqlite_file: String,
    /// If user IDs should be hased in the database, disables @mentioning.
    pub hash_ids: bool,
    /// Channel where messages should be posted.
    pub channel: u64,
    /// List of user IDs of users who have admin perms with the bot.
    pub admins: Vec<i64>,
    /// Directory where pictures are kept.
    pub picture_dir: String,
    /// List of pictures to acompany users.
    pub pictures: Vec<String>,
}

impl Default for CoreConfig {
    fn default() -> Self {
        Self { 
            token: Default::default(),
            log_level: "info".to_string(),
            log_dir: "logs".to_string(),
            sqlite_file: "data/db.sqlite".to_string(),
            hash_ids: true, 
            channel: 0,
            admins: Vec::new(),
            picture_dir: "data".to_string(),
            pictures: Vec::new(),
        }
    }
}

pub static CORE_CFG: OnceCell<CoreConfig> = OnceCell::new();

pub fn load_core_cfg(file: &String) {
    let f_ = std::fs::File::open(format!("{}", file));
    
    if f_.is_err() {
        CORE_CFG.set(CoreConfig::default()).expect("Could not set default values");
        println!("!! Using default config values becuase {} does not exist or could not be read !!", file);
        return;
    }
    
    let r = CORE_CFG.set(serde_yaml::from_reader(f_.unwrap()).expect("Could not read values."));
    
    if r.is_err() {
        CORE_CFG.set(CoreConfig::default()).expect("Could not set default values");
        println!("!! Using default config values becuase {} does not exist or could not be read !!", file)
    }
}