use rand::Rng;
use serde::ser::StdError;
use poise::serenity_prelude::ChannelId;
use poise::serenity_prelude::{self as serenity};
use rand::seq::SliceRandom;
use chrono::{NaiveDateTime, Utc};

use crate::{logging::{LogLevel, log}, config::CORE_CFG, db::user::*};

pub async fn handle_msg(ctx: &serenity::Context, msg: &serenity::Message) -> Result<(), Box<(dyn StdError + std::marker::Send + Sync + 'static)>> { 
    if !msg.is_private() || msg.author.bot {
        return Ok(());
    }
    
    log(LogLevel::Debug, "event_handlers", "handle_msg", msg.content.as_str());

    let cfg: &crate::config::CoreConfig = CORE_CFG.get().unwrap();
    
    let mut uusr = get_user_by_id(&msg.author.id.0);
    if uusr.is_none() {
        let fid = rand::thread_rng().gen_range(10000000..=99999999);
        let idx = rand::thread_rng().gen_range(0..cfg.pictures.len());
        uusr = create_user(&msg.author.id.0, None, fid, idx as i32);
        log(LogLevel::Info, "event_handlers", "handle_msg", format!("Create new user with fake ID {}", fid).as_str());
        
        
        if uusr.is_none() {
            let _ = msg.reply(&ctx.http, "Failed to create ID for you, please contact a sysadmin.").await;
            return Ok(())
        }
    }
    let mut usr = uusr.unwrap();

    if usr.is_banned {
        let _ = msg.reply(&ctx.http, "You have been banned from using this bot.").await;
        return Ok(())
    }

    let now: NaiveDateTime = Utc::now().naive_utc();
    if usr.when_generated.timestamp() + 86400 < now.timestamp()  {
        let fid: i64 = rand::thread_rng().gen_range(10000000..=99999999);
        let idx = rand::thread_rng().gen_range(0..cfg.pictures.len());
        log(LogLevel::Info, "event_handlers", "handle_msg", format!("Regenerate fake ID {}", fid).as_str());
        set_fake_id(&msg.author.id.0, fid, idx as i32);
        usr.fake_id = fid;
    }

    let new_channel = ChannelId::from(cfg.channel);
    let mut picture = cfg.pictures.choose(&mut rand::thread_rng()).unwrap();
    let idx_usize = usr.image_idx as usize;
    if idx_usize < cfg.pictures.len() {
        picture = &cfg.pictures[idx_usize];
    }
    
    let attachment_uri = format!("attachment://{}", picture);
    let icon_file_uri = format!("{}/{}", cfg.picture_dir, picture);

    let _msg: Result<serenity::Message, ::serenity::Error> = new_channel
                .send_message(&ctx.http, |m| {
                    m.content("")
                        .embed(|e| {
                            e.title("")
                                .description(&msg.content)
                                .color(0x9834eb)
                                .author(|a| {
                                    a.icon_url(attachment_uri);
                                    a.name(usr.fake_id.to_string())
                                })
                        })
                        .add_file(icon_file_uri.as_str())
                })
                .await;
    Ok(())
}

pub async fn handle_guild_join(_ctx: &serenity::Context, _guild: &serenity::Guild, _is_new: &bool) -> Result<(), Box<(dyn StdError + std::marker::Send + Sync + 'static)>> { 
    Ok(()) 
}