use poise::serenity_prelude::{self as serenity};
use std::str::FromStr;

mod config;
use config::{load_core_cfg, CORE_CFG};

mod logging;
use logging::{LogLevel, init_logger, log};

mod commands;
use commands::{notify::*, Data};

mod db;
use db::init_db;

mod handlers;
use handlers::*;



pub fn to_hex(data: Vec<u8>) -> String {
    let mut ret: String = "".to_string();

    for x in data {
        ret.push_str(format!("{:02X}", x).as_str());
    }
    ret
}

#[tokio::main]
async fn main() {
    load_core_cfg(&"config.yaml".to_string());    
    let cfg = CORE_CFG.get().unwrap();

    init_logger(&cfg.log_dir, LogLevel::from_str(cfg.log_level.as_str()).unwrap());
    if !init_db() {
        return;
    }

    let framework = poise::Framework::builder()
        .options(poise::FrameworkOptions {
            event_handler: |ctx: &serenity::Context, event: &poise::Event<'_>, _framework, _data: &Data| {
                Box::pin(async move {
                    log(LogLevel::Trace, "main", "event_handler", format!("Event Fired: {:?}", event.name()).as_str());
                    match event {
                        poise::Event::GuildCreate { guild, is_new } => handle_guild_join(ctx, guild, is_new).await,
                        poise::Event::Message { new_message } => handle_msg(ctx, new_message).await,
                        _ => Ok(()),
                    }
                })
            },
            commands: vec![notify()],
            ..Default::default()
        })
        .token(&cfg.token)
        .intents(serenity::GatewayIntents::non_privileged() | serenity::GatewayIntents::DIRECT_MESSAGES | serenity::GatewayIntents::GUILD_MESSAGES)
        .setup(|ctx, _ready, framework| {
            Box::pin(async move {
                poise::builtins::register_globally(ctx, &framework.options().commands).await?;
                Ok(Data {})
            })
        });

    log(LogLevel::Info, "boot", "main", "WhisperWolf3 boot");
    framework.run().await.unwrap();
}
